package windsock;

import windsock.*;
import java.net.*;
import java.io.*;

public class ClientHandler extends Thread {
    protected SocksSocket sock;

    public ClientHandler(Socket s) throws IOException {
	sock = new SocksSocket(s);
    }

    public void run() {
	final byte[] methods = { SocksSocket.METHOD_NOAUTH };

	try {
	    byte method = sock.getMethod(methods);

	    // to do: authentication

	    Request r = sock.getRequest();
	    Log.log(r);

	    Thread handler = null;
	    byte reply = SocksSocket.REPLY_SUCCESS;
	    try {
		switch (r.command) {
		    case SocksSocket.CMD_CONNECT:
			handler = new ConnectHandler(sock, r);
			break;

		    case SocksSocket.CMD_BIND:
			handler = new BindHandler(sock, r);
			sock.sendReply(r, reply);
			((BindHandler) handler).accept(r);
			break;

			/*
		    case SocksSocket.CMD_UDP_ASSOC:
			handler = new DatagramHandler(sock, r);
			break;
			*/

		    default:
			throw new SocksException("Unsupported command: " +
				r.command);
		}
	    } catch (SocksException e) {
		if (e.reply != -1) {
		    Log.err(e);
		    reply = (byte) e.reply;
		} else {
		    throw e;
		}
	    }

	    sock.sendReply(r, reply);
	    if (handler != null) {
		handler.run();
	    }

	} catch (Exception e) {
	    if (e instanceof IOException && e instanceof SocksException) {
		Log.err(e);
	    } else {
		e.printStackTrace();
	    }

	} finally {
	    if (sock != null) {
		try { sock.close(); } catch (IOException f) {}
	    }
	}
    }
}
